//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/CmdOptHandler.h,v $
// $Id: CmdOptHandler.h,v 1.2 2004/05/04 23:20:23 ah Exp $
//
// Description: interface of class IOCmdOption
//
//---------------------------------------------------------------------------

#ifndef AH_IO_CMD_OPT_HANDLER_H		// prevent multiple includes
#define AH_IO_CMD_OPT_HANDLER_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
USING_NAMESPACE(AH)

#include <string>
#include <vector>
USING_NAMESPACE(std)

BEGIN_NAMESPACE(AH)

// --- forward declarations ---

class IOCmdOption;

// --- types and classes ---

class IOCmdOptHandler
{
 public:
    /** default constructor. */
    IOCmdOptHandler() {}
    /** constructor. */
    IOCmdOptHandler(const std::vector<IOCmdOption*> newOpts)
	{ addOptions(newOpts); }
    /** destructor */
    ~IOCmdOptHandler() {}

    /** add one more option. */
    void addOption(IOCmdOption* newOpt) 
	{ opts.push_back(newOpt); }

    /** add more options. */
    void addOptions(const std::vector<IOCmdOption*> newOpts);

    /** handle options, return number of bad options. */
    int handleArgs(int argc, char* argv[],
		   std::vector<std::string>& remarg,
		   std::vector<std::string>& errwarn);

    /** return help text of all options. */
    std::string getHelpText(const std::string& lineHead);

    /** return short option string, for help text. */
    std::string getShortOptionText() const;

 private:
    /** return next entry if this is an argument. */
    std::string getNextOptArg(int& a, int argc, char* argv[]);
    /** handle one short argument. */
    int handleShortArg(int& a, int argc, char* argv[],
		       std::vector<std::string>& errwarn);
    /** handle one short argument. */
    int handleLongArg(int& a, int argc, char* argv[],
		      std::vector<std::string>& errwarn);

    std::vector<IOCmdOption*> opts;
    std::string short_opt;
};  // IOCmdOptHandler


END_NAMESPACE

#endif // !AH_IO_CMD_OPT_HANDLER_H







