//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/Convert.h,v $
// $Id: Convert.h,v 1.3 2003/03/14 22:04:59 ah Exp $
//
// Description: convert time into readable string
//
//---------------------------------------------------------------------------

#ifndef AH_IO_TIMEFORMAT_H		// prevent multiple includes
#define AH_IO_TIMEFORMAT_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Config/Types.h>
USING_NAMESPACE(AH)

#include <string>
USING_NAMESPACE(std)

// --- types and classes ---

BEGIN_NAMESPACE(AH)

/// class containing converter functions
/**
   <tt>IOConvert</tt> contains some usable converter functions.

   Please include  <em>AH/IO/Convert.h</em>
 */
class IOConvert
{
 public:
    // conversion: BIG endian <-> uint32  -------------------------------

    /// convert big endian data into 32bit unsigned integer
    /**
       Use <tt>BigEndCharsToUint32()</tt> to convert big endian data
       into a 32bit wide unsigned integer.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static uint32 BigEndCharsToUint32(const uint8 data[]);

    /// convert 32bit unsigned integer into big endian data
    /**
       Use <tt>Uint32ToBigEndChars()</tt> to convert a 32bit wide unsigned 
       integer into big endian data.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static void Uint32ToBigEndChars(uint32 val, uint8 data[]);

    // conversion: little endian <-> uint32 -----------------------------

    /// convert little endian data into 32bit unsigned integer
    /**
       Use <tt>LittleEndCharsToUint32()</tt> to convert little endian data
       into a 32bit wide unsigned integer.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static uint32 LittleEndCharsToUint32(const uint8 data[]);

    /// convert 32bit unsigned integer into little endian data
    /**
       Use <tt>Uint32ToLittleEndChars()</tt> to convert a 32bit wide unsigned 
       integer into little endian data.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static void Uint32ToLittleEndChars(uint32 val, uint8 data[]);

    // conversion: BIG endian <-> int32 ---------------------------------

    /// convert big endian data into 32bit signed integer
    /**
       Use <tt>BigEndCharsToInt32()</tt> to convert big endian data
       into a 32bit wide unsigned integer.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static int32 BigEndCharsToInt32(const uint8 data[]);

    /// convert 32bit signed integer into big endian data
    /**
       Use <tt>Int32ToBigEndChars()</tt> to convert a 32bit wide signed 
       integer into big endian data.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static void Int32ToBigEndChars(int32 val, uint8 data[]);

    // conversion: little endian <-> int32 ------------------------------

    /// convert little endian data into 32bit signed integer
    /**
       Use <tt>LittleEndCharsToInt32()</tt> to convert little endian data
       into a 32bit wide signed integer.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static int32 LittleEndCharsToInt32(const uint8 data[]);

    /// convert 32bit signed integer into little endian data
    /**
       Use <tt>Int32ToLittleEndChars()</tt> to convert a 32bit wide signed 
       integer into little endian data.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static void Int32ToLittleEndChars(int32 val, uint8 data[]);



    // conversion: BIG endian <-> uint16  -------------------------------

    /// convert big endian data into 16bit unsigned integer
    /**
       Use <tt>BigEndCharsToUint16()</tt> to convert big endian data
       into a 16bit wide unsigned integer.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static uint16 BigEndCharsToUint16(const uint8 data[]);

    /// convert 16bit unsigned integer into big endian data
    /**
       Use <tt>Uint16ToBigEndChars()</tt> to convert a 16bit wide unsigned 
       integer into big endian data.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static void Uint16ToBigEndChars(uint16 val, uint8 data[]);

    // conversion: little endian <-> uint16 -----------------------------

    /// convert little endian data into 16bit unsigned integer
    /**
       Use <tt>LittleEndCharsToUint16()</tt> to convert little endian data
       into a 16bit wide unsigned integer.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static uint16 LittleEndCharsToUint16(const uint8 data[]);

    /// convert 16bit unsigned integer into little endian data
    /**
       Use <tt>Uint16ToLittleEndChars()</tt> to convert a 16bit wide unsigned 
       integer into little endian data.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static void Uint16ToLittleEndChars(uint16 val, uint8 data[]);

    // conversion: BIG endian <-> int16 ---------------------------------

    /// convert big endian data into 16bit signed integer
    /**
       Use <tt>BigEndCharsToInt16()</tt> to convert big endian data
       into a 16bit wide unsigned integer.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static int16 BigEndCharsToInt16(const uint8 data[]);

    /// convert 16bit signed integer into big endian data
    /**
       Use <tt>Int16ToBigEndChars()</tt> to convert a 16bit wide signed 
       integer into big endian data.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static void Int16ToBigEndChars(int16 val, uint8 data[]);

    // conversion: little endian <-> int16 ------------------------------

    /// convert little endian data into 16bit signed integer
    /**
       Use <tt>LittleEndCharsToInt16()</tt> to convert little endian data
       into a 16bit wide signed integer.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static int16 LittleEndCharsToInt16(const uint8 data[]);

    /// convert 16bit signed integer into little endian data
    /**
       Use <tt>Int16ToLittleEndChars()</tt> to convert a 16bit wide signed 
       integer into little endian data.

       Please include  <em>AH/IO/Convert.h</em>
    */
    static void Int16ToLittleEndChars(int16 val, uint8 data[]);



    /// convert time value into readable string
    /**
       Use <tt>timeToString</tt> to convert a time value into a readable string.
       Currently only seconds are supported as time unit.

       Please include  <em>AH/IO/TimeFormat.h</em>
    */
    static string timeToString
    (
	/// time value (integer)
	unsigned int value, 
	/// time unit (ps, ns, us, ms, s, m, d, h, M, Y: default is s)
	const char* unit = "s"
    );

    /// convert time value into readable string
    /**
       Use <tt>timeToString</tt> to convert a time value into a readable string.
       Currently only seconds are supported as time unit.

       Please include  <em>AH/IO/TimeFormat.h</em>
    */
    static string timeToString
    (
	/// time value (floating point)
	double value, 
	/// time unit (ps, ns, us, ms, s, m, d, h, M, Y: default is s)
	const char* unit = "s"
    );

    /// convert time value into readable string in CD format
    /**
       Use <tt>timeToCdString</tt> to convert a time value into a readable string.

       Please include  <em>AH/IO/TimeFormat.h</em>
    */
    static string timeToCdString
    (
	/// time value (floating point)
	double value
    );
};

END_NAMESPACE

#endif // !AH_IO_TIMEFORMAT_H
