//---------------------------------------------------------------------------
// Copyright 1999 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/Riff/WaveFmt.h,v $
// $Id: WaveFmt.h,v 1.7 2004/04/20 22:25:30 ah Exp $
//
// Description: 
//
//---------------------------------------------------------------------------

#ifndef AH_IO_RIFF_WAVE_FMT_H		// prevent multiple includes
#define AH_IO_RIFF_WAVE_FMT_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Config/Types.h>
USING_NAMESPACE(AH)

#include <string>
#include <iostream>

// --- types and classes ---

BEGIN_NAMESPACE(AH)

class IORiffWaveFmt
{
    class Specific
    {
     public:
	Specific(uint16 k, uint32 s = 0) : key(k), dataSize(s) {}
	virtual ~Specific() {}
	virtual std::ostream& printFormat(std::ostream& out, const char* pre = 0);
	virtual std::ostream& printInfo(std::ostream& out, const char* pre = 0)
	    { return out; }
	virtual uint32 getDataSize() const { return dataSize; }
     private:
	uint16 key;
	uint32 dataSize;
    };

    class MsPcm: public Specific
    {
     public:
	MsPcm(unsigned char* data);
	MsPcm(unsigned char* data, uint16 samsize);
	virtual ~MsPcm() {}
	uint16 getSampleSize() const { return sampleSize; }
	virtual std::ostream& printFormat(std::ostream& out, const char* pre = 0);
	virtual std::ostream& printInfo(std::ostream& out, const char* pre = 0);
     private:
	uint16 sampleSize;
    };

    class Other: public Specific
    {
     public:
	Other(uint16 k, unsigned char* data);
	virtual ~Other() {}
	uint16 getSampleSize() const { return sampleSize; }
	virtual std::ostream& printInfo(std::ostream& out, const char* pre = 0);
     private:
	uint16 channels;
	uint32 sampleRate;
	uint32 dataRate;
	uint16 blockSize;
	uint16 sampleSize;
	uint16 extraSize;
    };

 public:
    /// wave format categories
    enum FormatCategory {
	UNKNOWN			= 0x0000,
	MS_PCM			= 0x0001,
	ADPCM			= 0x0002,
	IBM_CVSD		= 0x0005,
	ALAW			= 0x0006,
	MULAW			= 0x0007,
	OKI_ADPCM		= 0x0010,
	DVI_ADPCM		= 0x0011,
	DIGISTD			= 0x0015,
	DIGIFIX			= 0x0016,
	YAMAHA_ADPCM		= 0x0020,
	DSPGROUP_TRUESPEECH	= 0x0022,
	SONARC			= 0x0021,
	IBM_MULAW		= 0x0101,
	IBM_ALAW		= 0x0102,
	IBM_ADPCM		= 0x0103,
	CREATIVE_ADPCM		= 0x0200
    };

    /// constructor for input data
    IORiffWaveFmt(const unsigned char* data);

    /// constructor for output data
    IORiffWaveFmt(uint16 fmt, uint16 chans, uint32 sr, uint16 ss);

    /// destructor
    virtual ~IORiffWaveFmt();

    // access functions:
    /// return format key
    uint32 getFormatKey() const { return format; }
    /// return number of channels
    uint32 getChannels() const { return channels; }
    /// return sample rate in samples per second
    uint32 getSampleRate() const { return sampleRate; }
    /// return data rate in bytes per second
    uint32 getDataRate() const { return dataRate; }
    /// return block size
    uint32 getBlockSize() const { return blockSize; }
    /// return sample size in bits per sample
    uint16 getSampleSize() const;
    /// return pointer to data buffer
    const uint8* getDataBuffer() const { return data; }
    /// return data buffer size
    uint32 getDataSize() const;

    /// print information about this chunk
    virtual std::ostream& printInfo(std::ostream& out, const char* pre = 0);
    /// return format category as string
    static std::string getFormatName(FormatCategory fc);

 private:
    uint8*	heapdata;	// chunk on heap
    uint8*	data;		// pointer to chunk data
    uint16	format;
    uint16	channels;
    uint32	sampleRate;	// in samples per second
    uint32	dataRate;	// in bytes per second
    uint16	blockSize;
    Specific*	specData;	// specific data
};

END_NAMESPACE

#endif // !AH_IO_RIFF_WAVE_FMT_H




