//---------------------------------------------------------------------------
// Copyright 1999 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/RiffWaveFmt.c,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: RiffWaveFmt.c,v 1.8 2004/04/20 22:26:22 ah Exp $ (c) 1999 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/IO/Riff/WaveFmt.h>
#include <AH/IO/Convert.h>
#include <AH/Error/Sys.h>
#include <AH/Error/Intern.h>
USING_NAMESPACE(AH)

#include <cstdio>
#include <string>
#include <cerrno>
#include <iostream>
USING_NAMESPACE(std)

BEGIN_NAMESPACE(AH)

ostream& IORiffWaveFmt::Specific::printFormat(ostream& out, const char* pre)
{
    if (pre) out << pre;
    out << "format tag  = " << key << " (" 
	<< IORiffWaveFmt::getFormatName(FormatCategory(key)) << ")" << endl;
    return out;
}

IORiffWaveFmt::MsPcm::MsPcm(unsigned char* data)
    : Specific(0x0001, 2),
      sampleSize(IOConvert::LittleEndCharsToUint16(data))
{
}

IORiffWaveFmt::MsPcm::MsPcm(unsigned char* data, uint16 samsize)
    : Specific(0x0001, 2),
      sampleSize(samsize) 
{ 
    IOConvert::Uint16ToLittleEndChars(sampleSize, data);
}

ostream& IORiffWaveFmt::MsPcm::printFormat(ostream& out, const char* pre)
{
    if (pre) out << pre;
    out << "format      = Microsoft PCM" << endl;
    return out;
}

ostream& IORiffWaveFmt::MsPcm::printInfo(ostream& out, const char* pre)
{
    if (pre) out << pre;
    out << "sample size = " << sampleSize << " Bits" << endl;
    return out;
}

IORiffWaveFmt::Other::Other(uint16 k, unsigned char* data)
    : Specific(k, 16), 
      channels(IOConvert::LittleEndCharsToUint16(data)),
      sampleRate(IOConvert::LittleEndCharsToUint32(data+2)),
      dataRate(IOConvert::LittleEndCharsToUint32(data+6)),
      blockSize(IOConvert::LittleEndCharsToUint16(data+10)),
      sampleSize(IOConvert::LittleEndCharsToUint16(data+12)),
      extraSize(IOConvert::LittleEndCharsToUint16(data+14))
{
}

ostream& IORiffWaveFmt::Other::printInfo(ostream& out, const char* pre)
{
    if (pre) out << pre;
    out << "channels    = " << channels << endl;
    out << "sample rate = " << sampleRate << " Hz" << endl;
    out << "data rate   = " << dataRate << " Bytes/sec" << endl;
    out << "block size  = " << blockSize << " Bytes" << endl;
    out << "sample size = " << sampleSize << " Bits" << endl;
    out << "extra size  = " << extraSize << " Bytes" << endl;
    return out;
}

// constructor
IORiffWaveFmt::IORiffWaveFmt(const unsigned char* dt)
    : heapdata(0),
      data((uint8*)dt),
      format(IOConvert::LittleEndCharsToUint16(data)),
      channels(IOConvert::LittleEndCharsToUint16(data+2)),
      sampleRate(IOConvert::LittleEndCharsToUint32(data+4)),
      dataRate(IOConvert::LittleEndCharsToUint32(data+8)),
      blockSize(IOConvert::LittleEndCharsToUint16(data+12))
{
    switch (format)
    {
    default:     specData = new Other(format, data+14); break;
    case MS_PCM: specData = new MsPcm(data+14);          break;
    }
    if (!specData) throw ErrorSys(ENOMEM);
}

// constructor
IORiffWaveFmt::IORiffWaveFmt
(
    uint16 fmt, 
    uint16 chans, 
    uint32 samrate, 
    uint16 samsize
)
    : heapdata(new uint8[16]),
      data(heapdata),
      format(fmt),
      channels(chans),
      sampleRate(samrate)
{
    if (!heapdata) throw ErrorSys(ENOMEM);

    blockSize = channels * (samsize/8);
    dataRate  = sampleRate * blockSize;

    IOConvert::Uint16ToLittleEndChars(format, data);
    IOConvert::Uint16ToLittleEndChars(channels, data+2);
    IOConvert::Uint32ToLittleEndChars(sampleRate, data+4);
    IOConvert::Uint32ToLittleEndChars(dataRate, data+8);
    IOConvert::Uint16ToLittleEndChars(blockSize, data+12);
 
    switch (format)
    {
    default:     specData = new Other(format, data+14); break;
    case MS_PCM: specData = new MsPcm(data+14, samsize); break;
    }
    if (!specData) throw ErrorSys(ENOMEM);
}

// destructor
IORiffWaveFmt::~IORiffWaveFmt() 
{
    if (specData) delete specData;
    if (heapdata) delete[] heapdata;
}

// return data buffer size
uint32 IORiffWaveFmt::getDataSize() const
{
    uint32 size = 14;
    if (specData)
    {
	size += specData->getDataSize();
    }
    return size;
}

// return sample size in bits per sample
uint16 IORiffWaveFmt::getSampleSize() const
{
    if (specData)
    {
	const IORiffWaveFmt::MsPcm* pcm = 
	    dynamic_cast<const IORiffWaveFmt::MsPcm*>(specData);
	if (pcm) return pcm->getSampleSize();

	const IORiffWaveFmt::Other* oth = 
	    dynamic_cast<const IORiffWaveFmt::Other*>(specData);
	if (oth) return oth->getSampleSize();

	throw ErrorInternNotYetImpl(
	    getFormatName(FormatCategory(format)) + 
	    " Format");
    }
    throw ErrorInternNotYetImpl(
	getFormatName(FormatCategory(format)) + 
	" Format");

    return 0; // to keep old g++ quiet
}

ostream& IORiffWaveFmt::printInfo(ostream& out, const char* pre)
{
    if (specData) 
    {
	specData->printFormat(out, pre);
    }
    else
    {
	if (pre) out << pre;
	out << "format tag  = " << format << endl;
    }
    if (pre) out << pre;
    out << "channels    = " << channels << endl;
    if (pre) out << pre;
    out << "sample rate = " << sampleRate << "/sec" << endl;
    if (pre) out << pre;
    out << "data rate   = " << dataRate << " bytes/sec" << endl;
    if (pre) out << pre;
    out << "block size  = " << blockSize << endl;
    if (specData) specData->printInfo(out, pre);

    return out;
}

// return format category as string
string IORiffWaveFmt::getFormatName(FormatCategory fc)
{
    switch (fc)
    {
    case UNKNOWN:		return "UNKNOWN";
    case MS_PCM:		return "MS PCM";
    case ADPCM:			return "ADPCM";
    case IBM_CVSD:		return "IBM_CVSD";
    case ALAW:			return "ALAW";
    case MULAW:			return "MULAW";
    case OKI_ADPCM:		return "OKI_ADPCM";
    case DVI_ADPCM:		return "DVI_ADPCM";
    case DIGISTD:		return "DIGISTD";
    case DIGIFIX:		return "DIGIFIX";
    case YAMAHA_ADPCM:		return "YAMAHA_ADPCM";	
    case DSPGROUP_TRUESPEECH:	return "DSPGROUP_TRUESPEECH";
    case SONARC:		return "SONARC";
    case IBM_MULAW: 		return "IBM MULAW";
    case IBM_ALAW:  		return "IBM ALAW";
    case IBM_ADPCM:		return "IBM ADPCM";
    case CREATIVE_ADPCM:	return "CREATIVE_ADPCM";
    default:			return "unknown";
    }
}

END_NAMESPACE
