//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/TestCmdOption.c,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: TestCmdOption.c,v 1.3 2004/05/04 23:21:11 ah Exp $ (c) 1998 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/IO/CmdOptHandler.h>
#include <AH/IO/CmdOption.h>
#include <AH/Error/Class.h>
USING_NAMESPACE(AH)

#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <sstream>

USING_NAMESPACE(std)

// --- local implementation ---

static bool debug = false;

//----------------------------------------------------------------------
// --- global implementation ---
//----------------------------------------------------------------------


int main(int argc, char* argv[])
{
    cout << "Test of class IOCmdOptHandler:" << endl;

    if ((2 <= argc) && (0 == strcmp("-d", argv[1])))
	debug = true;

    int errnum = 0;
    int testnum = 0;

    try
    {
	IOCmdOption debopt('\0', "debug", IOCmdOption::SecondArgNone,
			   "enable debug mode");
	IOCmdOption veropt('v', "verbose", IOCmdOption::SecondArgAlways,
			   "increase verbose mode");
	IOCmdOption helpopt('h', "help", IOCmdOption::SecondArgNone,
			    "print this help meessage");

	IOCmdOptHandler hdl;
	hdl.addOption(&debopt);
	hdl.addOption(&veropt);
	hdl.addOption(&helpopt);
	std::vector<std::string> remargs;
	std::vector<std::string> errwarn;
	++testnum;
	errnum += hdl.handleArgs(argc, argv, remargs, errwarn);

	if (errwarn.size())
	{
	    for (unsigned int i = 0; i < errwarn.size(); ++i)
		cerr << "Eror/Warning: " << errwarn[i] << endl;
	}
	if (helpopt.isSet())
	{
	    cerr << "testcmdoption " << hdl.getShortOptionText() 
		 << " [arg1, arg2, ..]" << endl;
	    cerr << hdl.getHelpText("  ") << endl;
	}
	if (debopt.isSet())
	{
	    cerr << "debug mode is set " << debopt.isSet() 
		 << " time(s)" << endl;
	}
	if (veropt.isSet())
	{
	    cerr << "verbose mode is set " << veropt.isSet() 
		 << " time(s) to:";
	    for (unsigned int i = 0; i < veropt.getSecArg().size(); ++i)
		 cerr << " " << veropt.getSecArg()[i];
	    cerr << " (last = " << veropt.getLastSecArg() << ")";
	    cerr << endl;
	}
	for (unsigned int i = 0; i < remargs.size(); ++i)
	    cerr << "remaining arg " << i+1 << ": \"" << remargs[i]
		 << "\"" << endl;
    }
    catch(ErrorClass& err)
    {
	++errnum;
	cout << "Error: " << err.getMsg() << endl;
    }

    //----------------
    // print results:
    //----------------
    if (errnum)
	cout << "Stop. " << errnum << " error" 
	     << (errnum > 1 ? "s":"") << " found in " 
	     << testnum << " tests by " << argv[0] 
	     << "!" << endl;
    else
	cout << "Okay: no errors found in " << testnum << " tests." 
	     << endl;
    return (errnum ? 1 : 0);
}
