//---------------------------------------------------------------------------
// Copyright 1999 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/TestRiffWaveFmtChunk.c,v $
// $Id: TestRiffWaveFmtChunk.c,v 1.2 2004/04/20 22:26:22 ah Exp $
//
// Description: 
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: TestRiffWaveFmtChunk.c,v 1.2 2004/04/20 22:26:22 ah Exp $ (c) 1999 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/IO/Riff/WaveFmtChunk.h>	// my interface first, says Lakos
#include <AH/IO/Riff/StartChunk.h>	// my interface first, says Lakos
#include <AH/Error/Class.h>
#include <AH/IO/Smart/FilePtr.h>
#include <AH/IO/Smart/Buffer.h>
#include <AH/IO/CFormat.h>
USING_NAMESPACE(AH)

#include <cstdlib>
#include <unistd.h>
#include <string>
#include <cerrno>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <iostream>
USING_NAMESPACE(std)

typedef IOCFormat Cfm;

unsigned int execute_test
(
    const string& name,
    FILE* fp
)
{
    //------------------------
    // read 100k data of file :
    //------------------------
    size_t size = 102400;
    IOSmartBuffer<unsigned char> buf(size);
    if (0 == fread(buf(), buf.getSize(), 1, fp)) 
	throw ErrorSys(name);

    // get RIFF info from file data in buffer:
    IORiffStartChunk start(buf());
    start.printInfo(cout, "- start chunk: ");

    IORiffWaveFmtChunk fmt(start.nextChunk());
    fmt.printInfo(cout, "- fmt chunk: ");
    cout << "- fmt chunk: offset to next chunk = " 
	 << fmt.nextChunk() - start.nextChunk() << " bytes" << endl;
    cout << "- fmt chunk: offset to chunk data = " 
	 << fmt.chunkData() - start.nextChunk() << " bytes" << endl;

    IORiffChunk data(fmt.nextChunk());
    data.printInfo(cout, "- data: ");
    cout << "- data: offset to next chunk = " 
	 << data.nextChunk() - fmt.nextChunk() << " bytes" << endl;
    cout << "- data: offset to chunk data = " 
	 << data.chunkData() - fmt.nextChunk() << " bytes" << endl;

    // print out data:
    Cfm pos("%8.2f");
    Cfm dat("%6d");
    const int16* chl = (const int16*)data.chunkData();
    const int16* chr = (const int16*)(data.chunkData()+2);
    cerr << "# time/s Links Rechts Differenz" << endl; 
    do
    {
	if (*chl || *chr)
	{
	    const unsigned int shrink = 32;

	    if (((const unsigned char*)chl + 2 * shrink) < (buf() + size))
	    {
		int16 maxl = -32766;
		int16 maxr = -32766;
		int16 minl = 32767;
		int16 minr = 32767;
		for (unsigned int i = 0; i < shrink; i++, chl++, chr++)
		{
		    if (maxl < *chl) maxl = *chl;
		    if (minl > *chl) minl = *chl;
		    if (maxr < *chr) maxr = *chr;
		    if (minr > *chr) minr = *chr;
		}
		int lval = (maxl > -minl ? maxl : minl);
		int rval = (maxr > -minr ? maxr : minr);
		int position = (const unsigned char*)chl - data.chunkData();
//	        position /= 4 * shrink;
		cerr << pos << (double)position/(double)44100
		     << " " << dat << lval 
		     << " " << dat << rval 
		     << " " << dat << lval - rval 
		     << endl;
//		cerr << pos 
//		     << ((const unsigned char*)chl - data.chunkData()) / 4 
//		     << "   " << dat << *chl << " " << dat << *chr << endl;
	    }
	    else break;
	}
	else
	{
	    chl++;
	    chr++;
	}
    } while ((const unsigned char*)chl < (buf() + size));

    return 0;
}



int main(int argc, char* argv[])
{
    cerr << "Test of " << argv[0] << ":" << endl;

    unsigned int errnum = 0;
    for (int i=1; i < argc; i++)
    {
	string filename(argv[i]);
	cerr << "processing " << filename << " ..." << endl;

	IOSmartFilePtr file(filename, "r");
	errnum += execute_test(filename, file());
    }

    // print test results:
    cerr << argv[0] << ": ";
    if (errnum)
    {
	cerr << "failed with " << errnum << " error"
	     << (errnum > 1 ? "s" : "") << ". stop." << endl;
	return 1;;
    }
    cerr << "passed." << endl;
    return 0;
}
